///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#ifndef _LudumDare36_CardEntity_h_
#define _LudumDare36_CardEntity_h_

#include "ludumdare36.h"
#include "punch_cards.h"

namespace LudumDare36
{

	struct PunchCardTransform
	{
		explicit PunchCardTransform(const tbMath::Vector2& position = tbMath::Vector2::kZero, float orientation = 0.0f, float scale = 1.0f) :
			mPosition(position),
			mOrientation(orientation),
			mScale(scale, scale)
		{
		}

		tbMath::Vector2 mPosition;
		tbMath::Vector2 mScale;
		float mOrientation;
		
	};

	class SlidingCardEntity : public tbGame::Entity
	{
	public:
		static const PunchCardTransform kDeckTransform;
		static const PunchCardTransform kDiscardTransform;
		static const PunchCardTransform kRobotPlanTransform;
		static const PunchCardTransform kShuffleDeckTransform1;
		static const PunchCardTransform kShuffleDeckTransform2;
		static const PunchCardTransform kHandTransform[LudumDare36::kMaximumHandCount];
		static const PunchCardTransform kSlotTransform[LudumDare36::kMaximumSlotCount];

		SlidingCardEntity(const PunchCard& punchCard, const PunchCardTransform& start, const PunchCardTransform& final, 
			const tbGame::GameTimer& slideTimer);
		virtual ~SlidingCardEntity(void);

		static tbGraphics::Sprite GetSpriteForCard(const PunchCard& punchCard);
		static void RenderCard(const PunchCard& punchCard, const PunchCardTransform& transform);
		static bool IsPointContained(const tbMath::Vector2& point, const PunchCardTransform& transform);

	protected:
		virtual void OnAdded(void) override;
		virtual void OnRemoved(void) override;
		virtual void OnSimulate(void) override;
		virtual void OnUpdate(const float deltaTime) override;
		virtual void OnRender(void) const override;
		virtual void OnCollideWith(tbGame::Entity& otherEntity) override;

	private:
		const PunchCard mPunchCard;
		const PunchCardTransform mStartTransform;
		const PunchCardTransform mFinalTransform;
		const tbGame::GameTimer mInitialTimer;
		tbGame::GameTimer mSlideTimer;

		tbGraphics::Sprite mSprite;
	};

}; /* namespace LudumDare36 */

#endif /* _LudumDare36_CardEntity_h_ */
