///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "confirm_plan_entity.h"
#include "gameplay_scene.h"

const tbGame::GameTimer kMaximumSlideTime(500);

const tbMath::Vector2 kInPosition(640.0f, 360.0f);
const tbMath::Vector2 kOutPosition(640.0f, 1000.0f);

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::ConfirmPlanEntity::ConfirmPlanEntity(GameplayScene& gameplayScene) :
	tbGame::Entity("SlidingCardEntity"),
	mSprite("data/confirm_plans_box.png"),
	mGameplayScene(gameplayScene),
	mSlideTimer(0),
	mSlidingIn(false),
	mIsWinOrLoseBox(false),
	mDoInput(true),
	wave(0.0f)
{
	SetPosition((mSlidingIn) ? kInPosition : kOutPosition);

	mSprite.SetScroll(0.0f);
	mSprite.SetOrigin(tbGraphics::kAnchorCenter);
	AddGraphic(mSprite);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::ConfirmPlanEntity::ConfirmPlanEntity(GameplayScene& gameplayScene, bool reallyWinBox) :
	tbGame::Entity("SlidingCardEntity"),
	mSprite((true == reallyWinBox) ? "data/game_win_box.png" : "data/game_lose_box.png"),
	mGameplayScene(gameplayScene),
	mSlideTimer(0),
	mSlidingIn(false),
	mIsWinOrLoseBox(true),
	mDoInput(true),
	wave(0.0f)
{
	SetPosition((mSlidingIn) ? kInPosition : kOutPosition);

	mSprite.SetScroll(0.0f);
	mSprite.SetOrigin(tbGraphics::kAnchorCenter);
	AddGraphic(mSprite);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::ConfirmPlanEntity::~ConfirmPlanEntity(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::ConfirmPlanEntity::HideButton(bool immediate)
{
	if (true == mSlidingIn)
	{
		mSlideTimer = kMaximumSlideTime;
		mSlidingIn = false;
	}

	if (true == immediate)
	{
		mSlideTimer = 0;
		SetPosition(kOutPosition);
	}
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::ConfirmPlanEntity::ShowButton(void)
{
	if (false == mSlidingIn)
	{
		mSlideTimer = kMaximumSlideTime;
		mSlidingIn = true;
	}
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::ConfirmPlanEntity::OnSimulate(void)
{
	tbGame::Entity::OnSimulate();

	if (!mSlideTimer.IsZero())
	{
		mSlideTimer.DecrementStep();
		float percentage = 1.0f - mSlideTimer.GetPercentageOf(kMaximumSlideTime);

		if (true == mSlidingIn)
		{	//TODO: TIM: TurtleBrains: I think this interp is backwards WRT start/final.  Start should be 0% final 100%
			SetPosition(tbMath::Interpolation::SmoothStep(percentage, kInPosition, kOutPosition));
		}
		else
		{
			SetPosition(tbMath::Interpolation::SmoothStep(percentage, kOutPosition, kInPosition));
		}
	}
	else
	{
		SetPosition((mSlidingIn) ? kInPosition : kOutPosition);
	}
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::ConfirmPlanEntity::OnUpdate(const float deltaTime)
{
	wave += deltaTime;
	mSprite.SetScale(1.0f + sinf(wave * 4.0f) * 0.035f);

	tbGame::Entity::OnUpdate(deltaTime);

	if (false == mDoInput)
	{
		return;
	}

	if (true == mIsWinOrLoseBox)
	{
		if (true == tbGame::Input::IsKeyPressed(tbApplication::tbMouseLeft) ||
			true == tbGame::Input::IsKeyPressed(tbApplication::tbKeySpace) ||
			true == tbGame::Input::IsKeyPressed(tbApplication::tbKeyEscape) ||
			true == tbGame::Input::IsKeyPressed(tbApplication::tbKeyEnter))
		{
			mGameplayScene.OnFinishWinOrLose();
		}
		return;
	}

	if (true == tbGame::Input::IsKeyPressed(tbApplication::tbMouseLeft))
	{
		int mouseX(0);
		int mouseY(0);
		tbGame::Input::GetMousePosition(&mouseX, &mouseY);
		const tbMath::Vector2 mousePosition(static_cast<float>(mouseX), static_cast<float>(mouseY));
		const tbMath::Vector2 position(GetPosition());

		const float halfWidth(mSprite.GetWidth() / 2.0f);
		const float halfHeight(mSprite.GetHeight() / 2.0f);

		if (mousePosition.x > position.x - halfWidth && mousePosition.x < position.x + halfWidth &&
			mousePosition.y > position.y - halfHeight && mousePosition.y < position.y + halfHeight)
		{
			mGameplayScene.OnConfirmRobotPlan();
		}
	}
}

//--------------------------------------------------------------------------------------------------------------------//
