///
/// @file
/// @details This scene is the gameplay scene for the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#ifndef _LudumDare36_GameplayScene_h_
#define _LudumDare36_GameplayScene_h_

#include "ludumdare36.h"
#include "punch_cards.h"

#include "world_environment.h"
#include "confirm_plan_entity.h"

namespace LudumDare36
{

	class GameplayScene : public tbGame::GameScene
	{
	public:
		GameplayScene(void);
		virtual ~GameplayScene(void);

		bool IsAnyCardAnimating(bool checkOnlyTimer = true) const;

		void OnConfirmRobotPlan(void);
		void OnFinishWinOrLose(void);

	protected:
		virtual void OnSimulate(void) override;
		void SimulatePlayerPhase(void);
		void SimulateGamePhase(void);

		void ResetCameraPosition(void);
		void SimulateCamera(const tbMath::Vector2& mousePosition);

		virtual void OnUpdate(const float deltaTime) override;
		virtual void OnRender(void) const override;
		virtual void OnOpen(void) override;
		virtual void OnClose(void) override;

	private:
		friend class SlidingCardEntity;

		tbGame::InputAction mQuitGameAction;

		enum class GameCondition
		{
			kPlayingGame,
			kWonGame,
			kLostGame,
		};

		enum class GameTurnPhase
		{
			kBeginRound,
			
			kEnvironmentBegin,
			kEnvironmentTurn,

			kRobotBegin,
			kRobotTurn,

			kWinCheck,
			kEndRound,
		};

		enum class PlayerTurnPhase
		{
			kGameStartup,
			kShuffleDiscard,
			kDrawCards,
			kPlan,
			kConfirmPlan,
			kPowerUp,
			kRunRobot,
			kPowerDown,
		};

		tbGame::GameTimer mDrawCardTimer;

		PunchCardDeck mPunchCardDeck;
		PunchCardHand mPlayerHand;
		PunchCardPlanSlots mRobotPlanSlots;
		PunchDiscardPile mDiscardPile;
		PunchCardPile mRobotPlanPile;

		GameCondition mGameCondition;
		GameTurnPhase mGamePhase;
		PlayerTurnPhase mPlayerPhase;

		ConfirmPlanEntity mGameWinBox;
		ConfirmPlanEntity mGameLoseBox;
		ConfirmPlanEntity mConfirmButton;
		WorldEnvironment mWorld;

		tbMath::Vector2 mCameraDragDown;
		tbMath::Vector2 mCameraPosition;

		tbGame::InputAction mLeftClickAction;
		
		tbGame::InputAction mCamereResetAction;
		tbGame::InputAction mCameraMoveUpAction;
		tbGame::InputAction mCameraMoveDownAction;
		tbGame::InputAction mCameraMoveLeftAction;
		tbGame::InputAction mCameraMoveRightAction;
		tbGame::InputAction mRightClickAction;
		tbGame::InputAction mMiddleClickAction;

		size_t mCurrentLevel;
	};

}; /* namespace LudumDare36 */

#endif /* _LudumDare36_GameplayScene_h_ */
