///
/// @file
/// @details Entry point of the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "ludumdare36.h"
#include "scene_manager.h"

//--------------------------------------------------------------------------------------------------------------------//

int real_main(int argumentCount, const char* argumentValues[])
{
	tbGame::GameApplication gameApplication;
	gameApplication.SetWindowTitle(tb_string("Ancient Robots"));

	tbGraphics::theSpriteManager.LoadSpriteSheetFromFile("concept_art", "data/concept_kit/concept_art_sheet.json");
	tbGraphics::theSpriteManager.LoadSpriteSheetFromFile("punch_card_sheet", "data/punch_card_sheet.json");
	tbGraphics::theSpriteManager.LoadSpriteSheetFromFile("robot_sheet", "data/robot_sheet.json");
	tbAudio::theAudioManager.LoadEventTable("concept_audio", "data/concept_kit/concept_audio_events.json");
	tbAudio::theAudioManager.LoadEventTable("audio_events", "data/audio/audio_events.json");

	LudumDare36::SceneManager::CreateSceneManager();
#ifdef tb_release_build
	gameApplication.RunGame(LudumDare36::SceneManager::GetScene(LudumDare36::kSplashScene));
#else /* debug build */
	gameApplication.RunGame(LudumDare36::SceneManager::GetScene(LudumDare36::kGameplayScene));
#endif
	LudumDare36::SceneManager::DestroySceneManager();

	return 0;
}

//--------------------------------------------------------------------------------------------------------------------//

int main(const int argumentCount, const char* argumentValues[])
{
	tb_log_open("debug_log.txt", true);
	tb_log("Started Game: LudumDare36\n");

	int returnCode = tb_debug_project_entry_point_with(real_main, argumentCount, argumentValues);

	tb_log("Clean shutdown.\n");
	tb_log_close();
	return returnCode;
}

//--------------------------------------------------------------------------------------------------------------------//
