///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "punch_cards.h"
#include "sliding_cards.h"

#include <algorithm>

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchCardPile::PunchCardPile(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchCardPile::~PunchCardPile(void)
{
	while (!mCards.empty())
	{
		delete mCards.back().second;
		mCards.pop_back();
	}

}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::CardSlider* LudumDare36::PunchCardPile::AddCard(const PunchCard& punchCard)
{
	CardSlider* discardSlider = new CardSlider();
	mCards.push_back(std::pair<PunchCard, CardSlider*>(punchCard, discardSlider));
	return mCards.back().second;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::PunchCardPile::RemoveCard(const size_t& cardIndex)
{
	size_t previousIndex(cardIndex);
	delete mCards[cardIndex].second;

	for (size_t currentIndex(cardIndex + 1); currentIndex < mCards.size(); ++currentIndex)
	{
		mCards[previousIndex] = mCards[currentIndex];
		previousIndex = currentIndex;
	}

	mCards.pop_back();
}

//--------------------------------------------------------------------------------------------------------------------//

const LudumDare36::PunchCard& LudumDare36::PunchCardPile::GetCard(const size_t& atIndex) const
{
	return mCards[atIndex].first;
}

//--------------------------------------------------------------------------------------------------------------------//

const LudumDare36::CardSlider* LudumDare36::PunchCardPile::GetCardSlider(const size_t& atIndex) const
{
	return mCards[atIndex].second;
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::CardSlider* LudumDare36::PunchCardPile::GetCardSlider(const size_t& atIndex)
{
	return mCards[atIndex].second;
}

//--------------------------------------------------------------------------------------------------------------------//

size_t LudumDare36::PunchCardPile::Size(void) const
{
	return mCards.size();
}

//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchDiscardPile::PunchDiscardPile(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchDiscardPile::~PunchDiscardPile(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchCardDeck::PunchCardDeck(bool emptyDeck)
{
	mCards.reserve(512);

	if (false == emptyDeck)
	{
		CreateNewDeck();
	}
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchCardDeck::~PunchCardDeck(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::PunchCardDeck::CreateNewDeck(void)
{
	mCards.clear();

	AddCardsToDeck(PunchCard::kTurn90Left, 2);
	AddCardsToDeck(PunchCard::kTurn90Right, 2);
	AddCardsToDeck(PunchCard::kTurn180, 2);
	AddCardsToDeck(PunchCard::kMoveForward1, 3);
	AddCardsToDeck(PunchCard::kMoveForward2, 1);
	AddCardsToDeck(PunchCard::kMoveBackward1, 3);
	AddCardsToDeck(PunchCard::kMoveBackward2, 1);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchCard LudumDare36::PunchCardDeck::DrawCard(void)
{
	tb_error_if(mCards.empty(), "Error: Attempting to draw card from empty deck.");
	//tb_error_if(nullptr != mCards.back().second, "Error: Expected Deck not to use sliding effects.");

	const PunchCard punchCard(mCards.back().first);
	mCards.pop_back();
	return punchCard;
}

//--------------------------------------------------------------------------------------------------------------------//

int TurtleBrainsRandom(int i)
{
	return tbMath::RandomInt() % i;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::PunchCardDeck::ShuffleDeck(void)
{
	//std::random_shuffle(mCards.begin(), mCards.end(), TurtleBrainsRandom);
	std::random_shuffle(mCards.begin(), mCards.end());
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::PunchCardDeck::AddCardsToDeck(const PunchCard& punchCard, const size_t& count)
{
	for (size_t index(0); index < count; ++index)
	{
		//mCards.push_back(std::pair<PunchCard, CardSlider*>(punchCard, nullptr));
		AddCard(punchCard);
	}
}

//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchCardPlanSlots::PunchCardPlanSlots(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchCardPlanSlots::~PunchCardPlanSlots(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::CardSlider* LudumDare36::PunchCardPlanSlots::AddCard(const PunchCard& punchCard)
{
	tb_error_if(mCards.size() + 1 > LudumDare36::kMaximumSlotCount, "Error: Too many cards are in the planning slots.");
	return PunchCardPile::AddCard(punchCard);
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::PunchCardPlanSlots::RemoveCard(const size_t& cardIndex)
{
	size_t previousIndex(cardIndex);
	delete mCards[cardIndex].second;

	for (size_t currentIndex(cardIndex + 1); currentIndex < mCards.size(); ++currentIndex)
	{
		mCards[previousIndex] = mCards[currentIndex];
		mCards[previousIndex].second->PushEffect(CardSlidingEffect(SlidingCardEntity::kSlotTransform[currentIndex],
			SlidingCardEntity::kSlotTransform[previousIndex], 500));
		previousIndex = currentIndex;
	}

	mCards.pop_back();
}

//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchCardHand::PunchCardHand(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchCardHand::~PunchCardHand(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::CardSlider* LudumDare36::PunchCardHand::AddCard(const PunchCard& punchCard)
{
	tb_error_if(mCards.size() + 1 > LudumDare36::kMaximumHandCount, "Error: Too many cards are in the players hand.");
	return PunchCardPile::AddCard(punchCard);
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::PunchCardHand::RemoveCard(const size_t& cardIndex)
{
	size_t previousIndex(cardIndex);
	delete mCards[cardIndex].second;

	for (size_t currentIndex(cardIndex + 1); currentIndex < mCards.size(); ++currentIndex)
	{
		mCards[previousIndex] = mCards[currentIndex];
		mCards[previousIndex].second->PushEffect(CardSlidingEffect(SlidingCardEntity::kHandTransform[currentIndex],
			SlidingCardEntity::kHandTransform[previousIndex], 500));
		previousIndex = currentIndex;
	}

	mCards.pop_back();
}

//--------------------------------------------------------------------------------------------------------------------//
