///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#ifndef _LudumDare36_PunchCards_h_
#define _LudumDare36_PunchCards_h_

#include "ludumdare36.h"

namespace LudumDare36
{
	class CardSlider;

	enum class PunchCard
	{
		kFaceDown,
		kDeckPile,
		kDiscardPile,
		kEmptyPile,

		kDelayTurn,
		kTurn90Left,
		kTurn90Right,
		kTurn180,
		kMoveForward1,
		kMoveForward2,
		kMoveBackward1,
		kMoveBackward2,
	};
	
	typedef std::vector<std::pair<PunchCard, CardSlider*>> SlidingCardContainer;

	static const size_t kMaximumHandCount = 8;
	static const size_t kMaximumSlotCount = 5;

	class PunchCardPile
	{
	public:
		PunchCardPile(void);
		virtual ~PunchCardPile(void);

		virtual CardSlider* AddCard(const PunchCard& punchCard);
		virtual void RemoveCard(const size_t& cardIndex);

		void ClearCards(void) { mCards.clear(); }

		const PunchCard& GetCard(const size_t& atIndex) const;
		const CardSlider* GetCardSlider(const size_t& atIndex) const;
		CardSlider* GetCardSlider(const size_t& atIndex);
		size_t Size(void) const;

	protected:
		SlidingCardContainer mCards;
	};

	class PunchCardDeck : public PunchCardPile
	{
	public:
		PunchCardDeck(bool emptyDeck = false);
		~PunchCardDeck(void);

		void CreateNewDeck(void);
		PunchCard DrawCard(void);
		void ShuffleDeck(void);

	protected:
		void AddCardsToDeck(const PunchCard& punchCard, const size_t& count);

	private:
	};

	class PunchDiscardPile : public PunchCardPile
	{
	public:
		PunchDiscardPile(void);
		virtual ~PunchDiscardPile(void);
	};

	class PunchCardPlanSlots : public PunchCardPile
	{
	public:
		PunchCardPlanSlots(void);
		virtual ~PunchCardPlanSlots(void);

		virtual CardSlider* AddCard(const PunchCard& punchCard);
		virtual void RemoveCard(const size_t& cardIndex);
	};

	class PunchCardHand : public PunchCardPile //Specifically the players hand.
	{
	public:
		PunchCardHand(void);
		virtual ~PunchCardHand(void);

		virtual CardSlider* AddCard(const PunchCard& punchCard);
		virtual void RemoveCard(const size_t& cardIndex);
	};	

}; /* namespace LudumDare36 */

#endif /* _LudumDare36_PunchCards_h_ */
