///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#ifndef _LudumDare36_RobotEntity_h_
#define _LudumDare36_RobotEntity_h_

#include "ludumdare36.h"
#include "world_transform.h"
#include "punch_cards.h"

#include <queue>

namespace LudumDare36
{

	class WorldEnvironment;

	class GamePiece : public tbGame::Entity
	{
	public:
		GamePiece(void);
		virtual ~GamePiece(void);

		const WorldTransform& GetWorldTransform(void) const { return mWorldTransform; }
		void SetWorldTransform(const WorldTransform& worldTransform);
		
		void ResetGame(void);

		void BeginTurn(const WorldTransform& turnResult);
		void SimulateTurn(float turnPercentage);
		void EndTurn(void);

	protected:
		virtual void OnResetGame(void);

		virtual void OnBeginTurn(const WorldTransform& turnResult);
		virtual void OnSimulateTurn(float turnPercentage);
		virtual void OnEndTurn(void);

		WorldTransform mWorldTransform;
		tbGraphics::AnimatedSprite mSprite;

	private:
		tbGame::GameTimer mTurnTimer;
		WorldTransform mTurnResultTransform;
	};

	class RobotEntity : public GamePiece
	{
	public:
		RobotEntity(const WorldEnvironment& worldEnvironment);
		virtual ~RobotEntity(void);

		void HurtRobot(const int amount);
		void PerformInstantKill(void);
		bool IsAlive(void) const;
		int GetHealth(void) const { return mHealth; }

	protected:
		virtual void OnAdded(void) override;
		virtual void OnCollideWith(tbGame::Entity& otherEntity) override;

		virtual void OnResetGame(void) override;
		virtual void OnBeginTurn(const WorldTransform& turnResult) override;
		virtual void OnEndTurn(void) override;

	private:
		tbGraphics::AnimatedSprite mShieldBurstSprite;
		const WorldEnvironment& mWorldEnvironment;
		int mHealth;
	};

	class HealthDisplayEntity : public tbGame::Entity
	{
	public:
		static const int kMaximumHealth = 4;

		HealthDisplayEntity(const RobotEntity& robot);
		virtual ~HealthDisplayEntity(void);

	protected:
		virtual void OnSimulate(void) override;

	private:
		const RobotEntity& mRobot;
		int mLastHealth;
		tbGraphics::AnimatedSprite mHealthSprites[kMaximumHealth];
	};

}; /* namespace LudumDare36 */

#endif /* _LudumDare36_RobotEntity_h_ */
