///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "sliding_cards.h"
#include <algorithm>

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::CardSlidingEffect::CardSlidingEffect(const PunchCardTransform& start, const PunchCardTransform& final, const tbGame::GameTimer& slideTimer) :
	mStartTransform(start),
	mFinalTransform(final),
	mInitialTimer(slideTimer),
	mSlideTimer(slideTimer)
{
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::PunchCardTransform LudumDare36::CardSlidingEffect::Simulate(void)
{
	if (mSlideTimer.IsZero() || mSlideTimer.DecrementStep())
	{
		return mFinalTransform;
	}

	const float turnPercentage(1.0f - mSlideTimer.GetPercentageOf(mInitialTimer));

	PunchCardTransform transform = mStartTransform;
	transform.mPosition = tbMath::Interpolation::Linear(turnPercentage, mStartTransform.mPosition, mFinalTransform.mPosition);
	transform.mOrientation = tbMath::Interpolation::Linear(turnPercentage, mStartTransform.mOrientation, mFinalTransform.mOrientation);
	transform.mScale = tbMath::Interpolation::Linear(turnPercentage, mStartTransform.mScale, mFinalTransform.mScale);
	return transform;
}

//--------------------------------------------------------------------------------------------------------------------//

bool LudumDare36::CardSlidingEffect::IsSliding(void) const
{
	return !mSlideTimer.IsZero();
}

//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::CardSlider::CardSlider(void) :
	mCardTransform()
{
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::CardSlider::~CardSlider(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::CardSlider::Simulate(void)
{
	if (!mSlidingEffects.empty())
	{
		mCardTransform = mSlidingEffects.front().Simulate();
		if (false == mSlidingEffects.front().IsSliding())
		{
			mSlidingEffects.pop();
		}
	}
}

//--------------------------------------------------------------------------------------------------------------------//
