///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#ifndef _LudumDare36_SlidingCards_h_
#define _LudumDare36_SlidingCards_h_

#include "ludumdare36.h"
#include "punch_cards.h"
#include "card_entity.h"

#include <queue>

namespace LudumDare36
{

	class CardSlidingEffect
	{
	public:
		CardSlidingEffect(const PunchCardTransform& start, const PunchCardTransform& final, const tbGame::GameTimer& slideTimer);

		PunchCardTransform Simulate(void);
		bool IsSliding(void) const;

	private:
		const PunchCardTransform mStartTransform;
		const PunchCardTransform mFinalTransform;
		const tbGame::GameTimer mInitialTimer;
		tbGame::GameTimer mSlideTimer;
	};

	class CardSlider
	{
	public:
		CardSlider(void);
		~CardSlider(void);

		void Simulate(void);

		inline void PushEffect(const CardSlidingEffect& effect) { mSlidingEffects.push(effect); }
		inline bool IsAnimating(void) const { return mSlidingEffects.empty() ? false : true; }
		inline const PunchCardTransform& GetCardTransform(void) const { return mCardTransform; }

	private:
		std::queue<CardSlidingEffect> mSlidingEffects;
		PunchCardTransform mCardTransform;
	};


//	static const size_t kMaximumHandCount = 8;
//	static const size_t kMaximumSlotCount = 5;

	//class SlidingDeck
	//{
	//public:
	//	PunchCardDeck(bool emptyDeck = false);
	//	~PunchCardDeck(void);

	//	void CreateNewDeck(void);
	//	PunchCard DrawCard(void);
	//	void ShuffleDeck(void);

	//protected:
	//	CardContainer mCards;

	//	void AddCardsToDeck(const PunchCard& punchCard, const size_t& count);

	//private:
	//};

	//class SlidingDiscardPile
	//{
	//public:
	//	SlidingDiscardPile(void);
	//	~SlidingDiscardPile(void);

	//	void AddCard(const PunchCard& punchCard);
	//	void RemoveCard(const size_t& cardIndex);

	//protected:
	//	CardContainer mCards;
	//};

	//class SlidingHand : public PunchCardHand
	//{
	//public:
	//	PunchCardHand(void);
	//	~PunchCardHand(void);

	//	void AddCard(const PunchCard& punchCard) override;
	//	void RemoveCard(const size_t& cardIndex);

	//	inline const PunchCard& GetCard(const size_t& atIndex) const { return mCards[atIndex]; }

	//	inline size_t HandSize(void) const { return mCards.size(); }

	//protected:
	//	CardContainer mCards;
	//};	

}; /* namespace LudumDare36 */

#endif /* _LudumDare36_SlidingCards_h_ */
