///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#ifndef _LudumDare36_WorldEnvironment_h_
#define _LudumDare36_WorldEnvironment_h_

#include "ludumdare36.h"
#include "robot_entity.h"
#include "world_transform.h"

namespace LudumDare36
{

	class WorldEnvironment : public tbGraphics::GraphicList
	{
	public:
		static const size_t kWorldTileSize;
		static const tbGame::GameTimer kMaximumTurnExecutionTime;
		static const tbGame::GameTimer kMinimumDelayBetweenTurns;

		WorldEnvironment(tbGame::EntityManager& entityManager);
		virtual ~WorldEnvironment(void);

		void LoadWorld(const tbCore::tbString& levelPath);
		void Simulate(void);

		static WorldTransform GetTransformForInstruction(const WorldTransform& input, const PunchCard& punchCard);
		static std::vector<WorldTransform> GetAllTransformsForInstruction(const WorldTransform& input, const PunchCard& punchCard);

		bool CanPerformInstruction(const RobotEntity& robot, const PunchCard& punchCard) const;
		bool CanMoveToLocation(const RobotEntity& robot, const WorldTransform& transform) const;

		bool PerformInstantKillEffects(RobotEntity& robot) const;

		inline const RobotEntity& GetRobot(void) const { return mRobot; }
		bool IsRobotAtHome(void) const;

		void BeginEnvironmentTurn(void);
		bool SimulateEnvironmentTurn(void);

		void BeginRobotTurn(const PunchCard& punchCard);
		bool SimulateRobotTurn(void);

		void AnimateWater(void);

	private:
		tbGame::EntityManager& mEntityManager;
		tbGame::TileSystem mTileSystem;
		RobotEntity mRobot;

		tbGame::GameTimer mTurnDelayTimer;
		tbGame::GameTimer mEnvironmentTurnTimer;
		tbGame::GameTimer mRobotTurnTimer;
		tbGame::GameTimer mRobotHurtTimer;

		typedef std::vector<WorldTransform> WorldTransformContainer;
		WorldTransformContainer mHomePoints;

		WorldTransformContainer mImpassibleTiles;
		WorldTransformContainer mInstantKillTiles;
		WorldTransformContainer mSlidingTiles[4];
	};

}; /* namespace LudumDare36 */

#endif /* _LudumDare36_WorldEnvironment_h_ */
