///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#ifndef _LudumDare36_WorldTransform_h_
#define _LudumDare36_WorldTransform_h_

#include "ludumdare36.h"

namespace LudumDare36
{

	enum class WorldDirection
	{
		kNorth,
		kEast,
		kSouth,
		kWest
	};

	tbMath::Vector2 DirectionToVector(const WorldDirection& direction);
	float DirectionToOrienation(const WorldDirection& direction);

	struct WorldTransform
	{
		explicit WorldTransform(int column = 0, int row = 0, const WorldDirection& direction = WorldDirection::kNorth);
		explicit WorldTransform(int column, int row, const WorldDirection& direction, const float explicitOrientation);

		int mColumn;
		int mRow;
		WorldDirection mDirection;

		tbMath::Vector2 mPosition;
		tbMath::Vector2 mForward;
		float mOrientation;

		static WorldTransform Rotate90Left(const WorldTransform& input);
		static WorldTransform Rotate90Right(const WorldTransform& input);
		static WorldTransform Rotate180(const WorldTransform& input);
		static WorldTransform MoveWorld(const WorldTransform& input, const WorldDirection& direction, const int spaces);
		static WorldTransform MoveForward(const WorldTransform& input, int spaces);
	};

}; /* namespace LudumDare36 */

#endif /* _LudumDare36_WorldTransform_h_ */
